;+
; NAME:
;   rbsp_efw_make_l2_vsvy
;
; PURPOSE: Create L2 (level-2) CDF files from the Van Allen Probe EFW 
;			Vsvy data product. 
;
;
; CALLING SEQUENCE:
;
; ARGUMENTS:	sc -> Which probe? 'a' or 'b'
;				date -> ex, '2013-10-13'
;
; KEYWORDS:
;
;
; EXAMPLES:	rbsp_efw_make_l2_vsvy,'a','2012-10-13'
;
; NOTES: This program stuffs the following quantities into the skeleton CDF file.
;		V1-V6 -> single-ended potential quantities
;		(Vx+Vy)/2 -> opposing boom averages
;
;
;	The flag array to be inserted into CDF has the following 20 fields:
;		global_flag
;		eclipse
;		maneuver
;		efw_sweep
;		efw_deploy
;		v1_saturation
;		v2_saturation
;		v3_saturation
;		v4_saturation
;		v5_saturation
;		v6_saturation
;		Espb_magnitude
;		Eparallel_magnitude
;		magnetic_wake
;		undefined	
;		undefined	
;		undefined	
;		undefined	
;		undefined	
;		undefined	
;
;	This program only sets the values of the V1-V6_saturation. The other values
;	are set to N/A or to unknown
;
;
; HISTORY:
;   March 2013: Created by Aaron Breneman, University of Minnesota
;
;
; VERSION:
; $LastChangedBy: aaron breneman $
;-




;Test CDF dates
;31 Oct 2012	IP Shock and Pulsations (Large and Small Amplitude Signals)
;01 Nov 2012	Big Storm
;14 Nov 2012	Plasma Sheet Boundary layer Crossings
;13 Feb 2013	Dipolarizations
;17 Feb 2013	Dipolarizations


pro rbsp_efw_make_l2_vsvy,sc,date,folder=folder


	if ~keyword_set(folder) then folder ='~/Desktop/code/Aaron/RBSP/l2_processing_cribs/'

;The skeleton CDF file
	;source_file = 'rbsp'+sc+'_vsvy_efw-l2_00000000_v01.cdf'
	source_file = 'rbsp'+sc+'_efw-l2_vsvy-hires_00000000_v01.cdf'


	filematch = file_search(folder+source_file, count = filecount) ;check for existence of output file


;Load some data
	timespan,date
	rbspx = 'rbsp'+sc

	version = 'v01'

;Rename the skeleton file 
	filename = 'rbsp'+sc+'_efw-l2_vsvy-hires_'+strjoin(strsplit(date,'-',/extract))+'_'+version+'.cdf'
	file_copy,folder+source_file,folder+filename


;Load the survey data
	rbsp_load_efw_waveform,probe=sc,type='calibrated',datatype='vsvy'
	get_data,rbspx+'_efw_vsvy',data=vsvy 


;Not ready to release the V5, V6 values.
	vsvy.y[*,4] = -1.0e30
	vsvy.y[*,5] = -1.0e30


;Get the time structure for the flag values. These are not necessarily at the cadence
;of physical data.
	epoch_flag_times,date,5,epoch_qual,timevals



;Load HSK data
	rbsp_load_efw_hsk,probe=sc,/get_support_data

	get_data,rbspx+'_efw_vsvy_ccsds_data_BEB_config',data=beb
	get_data,rbspx+'_efw_vsvy_ccsds_data_DFB_config',data=dfb




;Get flag values

	na_val = -2    ;not applicable value
	fill_val = -1  ;value in flag array that indicates "dunno"
	good_val = 0   ;value for good data
	bad_val = 1    ;value for bad data
	maxvolts = 195. ;Max antenna voltage above which the saturation flag is thrown
	offset = 5   ;position in flag_arr of "v1_saturation" 

	tmp = replicate(0,n_elements(vsvy.x),6)

	;All the flag values for the entire EFW data set
	flag_arr = replicate(fill_val,n_elements(timevals),20)


	for i=0,5 do begin

		;flag bad values
		vbad = where(abs(vsvy.y[*,i]) ge maxvolts)
		if vbad[0] ne -1 then tmp[vbad,i] = bad_val

		;set good values
		vgood = where(abs(vsvy.y[*,i]) lt maxvolts)
		if vgood[0] ne -1 then tmp[vgood,i] = good_val

		;Interpolate the bad data values onto the pre-defined flag value times
		flag_arr[*,i+offset] = ceil(interpol(tmp[*,i],vsvy.x,timevals))

		;Set the unknown values. These quantities do have a direct relevance
		;to the Vsvy L2 data product
		flag_arr[*,0] = fill_val	;global_flag
		flag_arr[*,1] = fill_val	;eclipse
		flag_arr[*,2] = fill_val	;maneuver
		flag_arr[*,3] = fill_val	;efw_sweep
		flag_arr[*,4] = fill_val	;efw_deploy


		;Set the N/A values. These are not directly relevant to the quality
		;of the Vsvy product
		flag_arr[*,11] = na_val	;Espb_magnitude
		flag_arr[*,12] = na_val	;Eparallel_magnitude
		flag_arr[*,13] = na_val	;magnetic_wake
		flag_arr[*,14:19] = na_val  ;undefined values
		

	endfor



;Create average values of opposing antennas
	v1v2 = 0.5*(vsvy.y[*,0] + vsvy.y[*,1])
	v3v4 = 0.5*(vsvy.y[*,2] + vsvy.y[*,3])
	v5v6 = 0.5*(vsvy.y[*,4] + vsvy.y[*,5])


	;not ready to release V5 or V6 values
	v5v6[*] = -1.0e30
	
	
	vsvy_vavg = [[v1v2],[v3v4],[v5v6]]



;Make the time string
	times = vsvy.x
	times_hsk = beb.x

	epoch = tplot_time_to_epoch(times,/epoch16)
	epoch_hsk = tplot_time_to_epoch(times_hsk,/epoch16)




 
;Stuff values into the cdf skeleton structure

	cdfid = cdf_open(folder+filename)
	cdf_control, cdfid, get_var_info=info, variable='epoch'


	cdf_varput,cdfid,'epoch',epoch
	cdf_varput,cdfid,'epoch_hsk',epoch_hsk
	cdf_varput,cdfid,'epoch_qual',epoch_qual
	cdf_varput,cdfid,'vsvy',transpose(vsvy.y)
	cdf_varput,cdfid,'vsvy_vavg',transpose(vsvy_vavg)
	cdf_varput,cdfid,'vsvy_DFB_config',dfb.y
	cdf_varput,cdfid,'vsvy_BEB_config',beb.y
	cdf_varput,cdfid,'efw_qual',transpose(flag_arr)


	cdf_close, cdfid



	store_data,tnames(),/delete




;Load the newly filled CDF structure to see if it works

	cdf_leap_second_init
	cdf2tplot,files=folder + filename

	ylim,'vsvy',-20,20
	ylim,'vsvy_vavg',-20,20
	ylim,'vsvy_DFB_config',-2,2
	ylim,'vsvy_BEB_config',-2,2

	names = ['global_flag',$
		'eclipse',$
		'maneuver',$
		'efw_sweep',$
		'efw_deploy',$
		'v1_saturation',$
		'v2_saturation',$
		'v3_saturation',$
		'v4_saturation',$
		'v5_saturation',$
		'v6_saturation',$
		'Espb_magnitude',$
		'Eparallel_magnitude',$
		'magnetic_wake',$
		'undefined',$
		'undefined',$
		'undefined',$
		'undefined',$
		'undefined',$
		'undefined']

	split_vec,'efw_qual',suffix='_'+names

	ylim,'efw_qual_v*_saturation',-2,2
	ylim,1,-250,250
	tplot,['vsvy','vsvy_vavg','vsvy_DFB_config','vsvy_BEB_config','efw_qual']




end